from argparse import ArgumentParser

import logging
from typing import Optional


class Log:
    INSTANCE = None  # type: Optional[Log]
    LOGGER_NAME = 'obwsc'
    FORMAT_STRING = '[%(asctime)s][%(levelname)s][%(name)s] %(message)s'

    def __init__(self, level: int):
        self.formatter = logging.Formatter(Log.FORMAT_STRING)
        self.level = level

        self.logger = logging.getLogger(Log.LOGGER_NAME)
        self.logger.setLevel(level)
        self.logger.propagate = False

        self.handlers = []
        self._add_handler(self.logger, logging.StreamHandler())

    def _add_handler(self, logger, handler):
        handler.setLevel(self.level)
        handler.setFormatter(self.formatter)
        logger.addHandler(handler)
        self.handlers.append(handler)

    @staticmethod
    def add_args(parser: ArgumentParser):
        verbosity = parser.add_mutually_exclusive_group()
        verbosity.add_argument('-v', dest='info', action='store_true', help='Log info messages')
        verbosity.add_argument('-vv', dest='debug', action='store_true', help='Log debug messages')

    @staticmethod
    def setup(args):
        if args.info:
            level = logging.INFO
        elif args.debug:
            level = logging.DEBUG
        else:
            level = logging.ERROR

        if Log.INSTANCE is None:
            Log.INSTANCE = Log(level)

    def update_level(self, level):
        self.logger.setLevel(level)
        for handler in self.handlers:
            handler.setLevel(level)

    @staticmethod
    def debug(*args, **kwargs):
        assert Log.INSTANCE is not None
        Log.INSTANCE.logger.debug(*args, **kwargs)

    @staticmethod
    def info(*args, **kwargs):
        assert Log.INSTANCE is not None
        Log.INSTANCE.logger.info(*args, **kwargs)

    @staticmethod
    def warning(*args, **kwargs):
        assert Log.INSTANCE is not None
        Log.INSTANCE.logger.warning(*args, **kwargs)

    @staticmethod
    def error(*args, **kwargs):
        assert Log.INSTANCE is not None
        Log.INSTANCE.logger.error(*args, **kwargs)

    @staticmethod
    def exception(*args, **kwargs):
        assert Log.INSTANCE is not None
        Log.INSTANCE.logger.exception(*args, **kwargs)

    @staticmethod
    def fatal(*args, **kwargs):
        assert Log.INSTANCE is not None
        Log.INSTANCE.logger.fatal(*args, **kwargs)


from argparse import ArgumentParser
from typing import Callable, Optional, List

import os
import sys

import toml

from obwsc.log import Log


class Options:
    @staticmethod
    def _get_default_working_dir() -> str:
        result = os.path.dirname(sys.argv[0])

        # Handle the case where we are compiled into a standalone package
        if sys.platform == 'darwin' and 'Contents/MacOS' in result:
            # We are inside af a macOS bundle
            result = os.path.abspath(os.path.join(result, '..', '..', '..'))
        else:
            result = os.getcwd()

        return result

    @staticmethod
    def parse(name: str, extra_args_fn: Optional[Callable[[ArgumentParser], None]] = None,
              cmd_args: Optional[List[str]] = None):
        parser = ArgumentParser(name)

        Log.add_args(parser)

        parser.add_argument('--config', '-c',
                            default=os.path.join(Options._get_default_working_dir(), 'config.toml'),
                            required=False, help='Path to the TOML configuration file')
        parser.add_argument('--host', '-a', default=None, type=str, required=False,
                            help='Host address, the OBS WS is running on')
        parser.add_argument('--port', '-p', default=None, type=int, required=False,
                            help='Port number, the OBS WS is running on')
        parser.add_argument('--password', '-s', default=None, type=str, required=False, help='OBS Web Socket Password')

        if extra_args_fn is not None:
            extra_args_fn(parser)

        args = parser.parse_args(args=cmd_args)

        Log.setup(args)

        if os.path.exists(args.config) and os.path.isfile(args.config):
            config = toml.load(args.config)
        else:
            config = {'obs': {'host': None, 'port': None, 'password': None}}

        def set_if_present(key: str, value):
            if value is not None:
                config['obs'][key] = value

        set_if_present('host', args.host)
        set_if_present('port', args.port)
        set_if_present('password', args.password)

        if config['obs']['host'] is None or config['obs']['port'] is None:
            raise RuntimeError(
                'Host and port values are required (either via config.toml or as command line arguments)')

        return config, args



from obwsc.commands.event_based_command import EventBasedCommand

from obwsc.log import Log


class StopRecord(EventBasedCommand):
    def __init__(self, config):
        super().__init__(config)
        self.events.callback.register(self.on_record_state_changed)

    @staticmethod
    def get_parser_name():
        return 'stop-record'

    @staticmethod
    def get_help():
        return 'Stop an OBS recording'

    def on_record_state_changed(self, event):
        if event.output_state == 'OBS_WEBSOCKET_OUTPUT_STOPPED':
            self.done.set()

    def execute(self):
        status = self.ws.get_record_status()
        if not status.output_active:
            Log.debug('Recording is already stopped')
            self.done.set()
            return

        self.ws.stop_record()
        self.done.wait()



from obwsc.commands.event_based_command import EventBasedCommand
from obsws_python.error import OBSSDKRequestError

import sys

from obwsc.log import Log


class ResumeRecord(EventBasedCommand):
    def __init__(self, config):
        super().__init__(config)
        self.events.callback.register(self.on_record_state_changed)

    @staticmethod
    def get_parser_name():
        return 'resume-record'

    @staticmethod
    def get_help():
        return 'Resume a paused OBS recording'

    def on_record_state_changed(self, event):
        if event.output_state == 'OBS_WEBSOCKET_OUTPUT_RESUMED':
            self.done.set()

    def _restart_macos_inputs(self):
        # On macOS the "screen capture" inputs get "broken" after locking the screen, but luckily OBS provides a button
        # for restarting the capture.
        # In this function we are iterating over all inputs, where this button is present and ask OBS to press it.
        if sys.platform != 'darwin':
            Log.debug(f'Skipping screen capture restarts: not on macOS: {sys.platform}')
            return

        resp = self.ws.get_input_list('screen_capture')
        for capture in resp.inputs:
            try:
                self.ws.press_input_properties_button(capture['inputName'], "reactivate_capture")
            except OBSSDKRequestError:
                Log.debug(f'Screen capture reactivation failed for "{capture["inputName"]}"')

    def execute(self):
        status = self.ws.get_record_status()
        if not status.output_active:
            raise RuntimeError('Recording is not active')

        if not status.output_paused:
            Log.debug('Recording is already running')
            self.done.set()
        else:
            self.ws.resume_record()

        self.done.wait()

        self._restart_macos_inputs()



import argparse
import time

from obwsc.commands.event_based_command import EventBasedCommand
from obwsc.commands.stop_record import StopRecord
from obwsc.commands.start_record import StartRecord
from obwsc.commands.set_current_profile import SetCurrentProfile
from obwsc.commands.set_current_scene_collection import SetCurrentSceneCollection
from obwsc.log import Log


class SwitchProfileAndSceneCollection(EventBasedCommand):
    def __init__(self, config, profile: str, collection: str):
        super().__init__(config)
        self.config = config
        self.target_profile = profile
        self.target_collection = collection

    @staticmethod
    def get_parser_name():
        return 'switch-profile-and-scene-collection'

    @staticmethod
    def get_help():
        return 'Switch the current OBS profile and scene collection, stopping the recording if necessary'

    @staticmethod
    def add_arguments(parser: argparse.ArgumentParser):
        parser.add_argument('profile', type=str, help='The name of the OBS profile to switch to')
        parser.add_argument('scene', type=str, help='The name of the OBS scene collection to switch to')

    @staticmethod
    def parse_arguments(args) -> dict:
        return {'profile': args.profile, 'collection': args.scene}

    def execute(self):
        profiles = self.ws.get_profile_list()
        if self.target_profile not in profiles.profiles:
            raise RuntimeError(f'Profile "{self.target_profile}" does not exist')

        change_profile = profiles.current_profile_name != self.target_profile

        scenes = self.ws.get_scene_collection_list()
        if self.target_collection not in scenes.scene_collections:
            raise RuntimeError(f'Scene collection "{self.target_collection}" does not exist')

        change_scene = scenes.current_scene_collection_name != self.target_collection

        if not change_profile and not change_scene:
            Log.debug('Profile and scene collection already active')
            return

        if change_profile:
            StopRecord(self.config).execute()
            time.sleep(1)  # OBS Reports an old status even after generating a "recording stopped" event
            SetCurrentProfile(self.config, self.target_profile).execute()

        if change_scene:
            SetCurrentSceneCollection(self.config, self.target_collection).execute()

        StartRecord(self.config).execute()



import argparse

from obwsc.commands.event_based_command import EventBasedCommand

from obwsc.log import Log


class SetCurrentSceneCollection(EventBasedCommand):
    def __init__(self, config, collection: str):
        super().__init__(config)
        self.events.callback.register(self.on_current_scene_collection_changed)
        self.target_collection = collection

    @staticmethod
    def get_parser_name():
        return 'set-current-scene-collection'

    @staticmethod
    def get_help():
        return 'Set the current OBS scene collection'

    @staticmethod
    def add_arguments(parser: argparse.ArgumentParser):
        parser.add_argument('collection', type=str, help='The name of the OBS scene collection to switch to')

    @staticmethod
    def parse_arguments(args) -> dict:
        return {'collection': args.collection}

    def on_current_scene_collection_changed(self, event):
        if event.scene_collection_name == self.target_collection:
            self.done.set()

    def execute(self):
        scenes = self.ws.get_scene_collection_list()
        if self.target_collection not in scenes.scene_collections:
            raise RuntimeError(f'Scene collection "{self.target_collection}" does not exist')

        if scenes.current_scene_collection_name == self.target_collection:
            Log.debug('Scene collection already active')
            return

        self.ws.set_current_scene_collection(self.target_collection)
        self.done.wait()
